@@ -39,7 +39,7 @@ use Email::Stuff     ();
 
 use vars qw{$VERSION};
 BEGIN {
-	$VERSION = '0.91';
+	$VERSION = '0.92';
 }
 
 use Object::Tiny qw{
@@ -404,27 +404,7 @@ sub view_list {
 sub view_promote {
 	my $self = shift;
 	$self->admins_only or return 1;
-
-	# Prepare the user list
-	my @users = $self->all_users;
-	my $list  = '';
-	my $cgi   = $self->cgi;
-	$cgi->param( a => 'm');
-	foreach my $user ( @users ) {
-		my $item = $self->cgi->escapeHTML($user->username);
-		if ( $self->is_user_admin($user) ) {
-			$item = $self->cgi->b($item);
-		} else {
-			$cgi->param( e => $item );
-			$item = $self->cgi->a( {
-				-href => $cgi->self_url,
-				}, $item );
-		}
-		$list .= $item . $self->cgi->br . "\n";
-	}
-
-	# Show the page
-	$self->{args}->{users} = $list;
+	$self->{args}->{users} = $self->user_checkbox_list;
 	$self->print_template(
 		$self->html_promote,
 	);
@@ -434,31 +414,49 @@ sub action_promote {
 	my $self = shift;
 	$self->admins_only or return 1;
 
-	# Does the account exist
-	my $email = _STRING($self->cgi->param('e'));
-	unless ( $email ) {
-		return $self->error("You did not enter an email address");
-	}
-	my $user = $self->auth->lookup_user($email);
-	unless ( $user ) {
-		return $self->error("No account for that email address");
+    # Which accounts are we promoting
+	my @accounts = $self->cgi->param('e');
+	unless ( @accounts ) {
+		return $self->error("You did not select an account");
 	}
 
-	# We can't operate on admins
-	if ( $self->is_user_admin($user) ) {
-		return $self->error("You cannot control other admins");
-	}
+    # Check all the proposed promotions first
+    my @users = ();
+    foreach ( @accounts ) {
+        my $account = _STRING($_);
+        unless ( $account ) {
+            return $self->error("Missing, invalid, or corrupt email address");
+        }
 
-	# Thus, they exist and are not an admin.
-	# So we now upgrade them to an admin.
-	$user->extra_info('admin');
+    	# Does the account exist
+        my $user = $self->auth->lookup_user($account);
+        unless ( $user ) {
+            return $self->error("The account '$account' does not exist");
+        }
 
-	# Send the promotion email
-	$self->{args}->{email} = $user->username;
-	$self->send_promote($user);
+    	# We can't operate on admins
+	    if ( $self->is_user_admin($user) ) {
+		    return $self->error("You cannot control admin account '$account'");
+	    }
+
+        push @users, $user;
+    }
+
+    # Apply the promotions and send mails
+    foreach my $user ( @users ) {
+    	$user->extra_info('admin');
+
+    	# Send the promotion email
+	    $self->{args}->{email} = $user->username;
+	    $self->send_promote($user);
+    }
 
 	# Show the "Promoted ok" page
-	$self->view_message("Promoted account $email to admin");
+	return $self->view_message(
+        join( "\n", map {
+            "Promoted account " . $_->username . " to admin"
+        } @users )
+    );
 }
 
 sub send_promote {
@@ -475,27 +473,7 @@ sub send_promote {
 sub view_delete {
 	my $self = shift;
 	$self->admins_only or return 1;
-
-	# Prepare the user list
-	my @users = $self->all_users;
-	my $list  = '';
-	my $cgi   = $self->cgi;
-	$cgi->param( a => 'e');
-	foreach my $user ( @users ) {
-		my $item = $self->cgi->escapeHTML($user->username);
-		if ( $self->is_user_admin($user) ) {
-			$item = $self->cgi->b($item);
-		} else {
-			$cgi->param( e => $item );
-			$item = $self->cgi->a( {
-				-href => $cgi->self_url,
-				}, $item );
-		}
-		$list .= $item . $self->cgi->br . "\n";
-	}
-
-	# Show the page
-	$self->{args}->{users} = $list;
+	$self->{args}->{users} = $self->user_checkbox_list;
 	$self->print_template(
 		$self->html_delete,
 	);
@@ -505,27 +483,45 @@ sub action_delete {
 	my $self = shift;
 	$self->admins_only or return 1;
 
-	# Does the account exist
-	my $email = _STRING($self->cgi->param('e'));
-	unless ( $email ) {
-		return $self->error("You did not enter an email address");
-	}
-	my $user = $self->auth->lookup_user($email);
-	unless ( $user ) {
-		return $self->error("No account for that email address");
+    # Which accounts are we deleting
+	my @accounts = $self->cgi->param('e');
+	unless ( @accounts ) {
+		return $self->error("You did not select an account");
 	}
 
-	# We can't operate on admins
-	if ( $self->is_user_admin($user) ) {
-		return $self->error("Admins cannot control other admins");
-	}
+    # Check all the proposed promotions first
+    my @users = ();
+    foreach ( @accounts ) {
+        my $account = _STRING($_);
+        unless ( $account ) {
+            return $self->error("Missing, invalid, or corrupt email address");
+        }
 
-	# Thus, they exist and are not an admin.
-	# So we now delete the user.
-	$self->auth->delete_user($user);
+    	# Does the account exist
+        my $user = $self->auth->lookup_user($account);
+        unless ( $user ) {
+            return $self->error("The account '$account' does not exist");
+        }
+
+    	# We can't operate on admins
+	    if ( $self->is_user_admin($user) ) {
+		    return $self->error("You cannot control admin account '$account'");
+	    }
+
+        push @users, $user;
+    }
+
+	# Delete the accounts
+    foreach my $user ( @users ) {
+	    $self->auth->delete_user($user);
+    }
 
 	# Show the "Deleted ok" page
-	$self->view_message("Deleted account $email");
+	return $self->view_message(
+        join( "\n", map {
+            "Deleted account " . $_->username
+        } @users )
+    );
 }
 
 sub view_change {
@@ -616,6 +612,7 @@ sub send_new {
 sub view_message {
 	my $self = shift;
 	$self->{args}->{message} = CGI::escapeHTML(shift);
+    $self->{args}->{message} =~ s/\n/<br \/>/g;
 	$self->print_template(
 		$self->html_message,
 	);
@@ -653,8 +650,8 @@ sub template {
 	my $self = shift;
 	my $html = shift;
 	my $args = shift || $self->args;
+	# Allow up to 10 levels of recursion
 	foreach ( 0 .. 10 ) {
-		# Allow up to 10 levels of recursion
 		$html =~ s/\[\%\s+(\w+)\s+\%\]/$args->{$1}/g;
 	}
 	return $html;
@@ -724,6 +721,36 @@ sub admins_only {
 	return 1;
 }
 
+sub user_checkbox_list {
+	my $self = shift;
+
+	# Prepare the user list
+	my $list  = '';
+	foreach my $user ( $self->all_users ) {
+		my $item = $self->cgi->escapeHTML($user->username);
+		if ( $self->is_user_admin($user) ) {
+			$list .= $self->cgi->b(
+				$self->cgi->checkbox(
+					-name     => '_',
+					-value    => $user->username,
+					-checked  => undef,
+					-disabled => undef,
+					-label    => $user->username,
+				)
+			);
+		} else {
+			$list .= $self->cgi->checkbox(
+				-name     =>  'e',
+				-value    => $user->username,
+				-label    => $user->username,
+			);
+		}
+		$list .= $self->cgi->br . "\n";
+	}
+
+	return $list;
+}
+
 
 
 
@@ -776,8 +803,10 @@ sub html_public { <<'END_HTML' }
 <p><a href="?a=c">I want to change my password</a></p>
 <h2>Admin</h2>
 <form method="post" name="f" action="">
-<p><input type="text" name="_e" size="30"> Email</p>
-<p><input type="text" name="_p" size="30"> Password</p>
+<p>Email</p>
+<p><input type="text" name="_e" size="30"></p>
+<p>Password</p>
+<p><input type="text" name="_p" size="30"></p>
 <p><input type="submit" name="s" value="Login"></p>
 </form>
 <hr>
@@ -823,11 +852,10 @@ sub html_forgot { <<'END_HTML' }
 <input type="hidden" name="a" value="r">
 <p>I can't tell you what your current password is, but I can send you a new one.</p>
 <p>&nbsp;</p>
-<p>What is your email address? <input type="text" name="e" size="30"> <input type="submit" name="s" value="Email me a new password"></p>
+<p>Email Address</p>
+<p><input type="text" name="e" size="30"></p>
+<p><input type="submit" name="s" value="Email me a new password"></p>
 </form>
-<p>&nbsp;</p>
-<hr>
-[% HOME %]
 </body>
 </html>
 END_HTML
@@ -860,8 +888,6 @@ sub html_change { <<'END_HTML' }
 </table>
 <p>Hit the button when you are ready to go <input type="submit" name="s" value="Change my password"></p>
 </form>
-<hr>
-[% HOME %]
 <script language="JavaScript">
 document.f.e.focus();
 </script>
@@ -893,8 +919,12 @@ sub html_promote { <<'END_HTML' }
 <html>
 [% HEAD %]
 <body>
-<h2>Click to Promote Account</h2>
+<h2>Select Account(s) to Promote</h2>
+<form name="f" action="">
+<input type="hidden" name="a" value="m">
 [% users %]
+<input type="submit" name="s" value="Promote">
+</form>
 </body>
 </html>
 END_HTML
@@ -908,8 +938,12 @@ sub html_delete { <<'END_HTML' }
 <html>
 [% HEAD %]
 <body>
-<h2>Click to Delete Account</h2>
+<h2>Select Account(s) to Delete</h2>
+<form name="f" action="">
+<input type="hidden" name="a" value="e">
 [% users %]
+<input type="submit" name="s" value="Delete">
+</form>
 </body>
 </html>
 END_HTML
@@ -926,7 +960,8 @@ sub html_new { <<'END_HTML' }
 <h2>Admin - Add a new user</h2>
 <form method="post" name="f">
 <input type="hidden" name="a" value="a">
-<p>Email <input type="text" name="e" size="30"></p>
+<p>Email</p>
+<p><input type="text" name="e" size="30"></p>
 <p><input type="submit" name="s" value="Add New User"></p>
 </form>
 </body>
